#!/usr/bin/env python

# Import the system library
import sys
# Import the python Argument parser
import argparse
# Import the maths library
import math as math

# A class for the golden plover population model
class GoldenPloverPopModel (object):

    # A function to parse the input parameters file.
    def parseParameterFile(self, inputFile):
        # A string to store the input parameters to
        # be outputted in the output file.
        paramsStr = "## Input Parameters to the model.\n"
        # Open the input parameter file.
        parameterFile = open(inputFile, 'r')
        # Create a dictionary object to store the 
        # input parameters.
        params = dict()
        # Loop through each line of the input 
        # text file.
        for line in parameterFile:
            # Strip any white space either
            # side of the text.
            line = line.strip()
            # Add the line to the output 
            # parameters file.
            paramsStr += "# " + line + "\n"
            # Split the line on the '=' symbol
            paramVals = line.split("=", 1)
            # Find the known parameters and
            # convert the input string to the 
            # correct data type (i.e., float or int).
            if paramVals[0] == "numOfYears":
                params[paramVals[0]] = int(paramVals[1])
            elif paramVals[0] == "initalAdultPairPop":
                params[paramVals[0]] = int(paramVals[1])
            elif paramVals[0] == "winterSurvivalRate":
                params[paramVals[0]] = float(paramVals[1])
            elif paramVals[0] == "averageEggsPerPair":
                params[paramVals[0]] = float(paramVals[1])
            elif paramVals[0] == "averageFledgelingsPerPair":
                params[paramVals[0]] = float(paramVals[1])
            elif paramVals[0] == "predatorControl":
                if paramVals[1].lower() == "false":
                    params[paramVals[0]] = False
                elif paramVals[1].lower() == "true":
                    params[paramVals[0]] = True
                else:
                    print "predatorControl must be either True or False."
                    sys.exit()
            elif paramVals[0] == "numOfFledgelings":
                params[paramVals[0]] = int(paramVals[1])
            elif paramVals[0] == "numOfFledgelingsYearOld":
                params[paramVals[0]] = int(paramVals[1])
            elif paramVals[0] == "fledgelingsSurvivePredatorsCtrl":
                params[paramVals[0]] = float(paramVals[1])
            elif paramVals[0] == "fledgelingsSurvivePredatorsNoCtrl":
                params[paramVals[0]] = float(paramVals[1])
            else:
                # If parameter is not known then just store as
                # a string.
                params[paramVals[0]] = paramVals[1]
        # Return the parameters and parameters string
        return params, paramsStr            
    
    # The run function controlling the overall order
    # of when things run.
    def run(self, inputFile):
        # Provide user feedback to the user.
        print "Parse Input File."
        # Call the function to parse the input file.
        params, paramsStr = self.parseParameterFile(inputFile)
        # Print he parameters.
        print params
        

# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, help="Specify the input image file.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: No input image file provided."
        sys.exit()
        
    obj = GoldenPloverPopModel()
    obj.run(args.input)
