#!/usr/bin/env python

# Import the python Argument parser
import argparse
# Import the RIOS applier interface
from rios import applier
from rios import cuiprogress

# Define the applier function
def mutliplyByValue(info, inputs, outputs, otherargs):
    # Multiple the image1 by the scale factor
    outputs.outimage = inputs.image1 * otherargs.scale

# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Define the argument for specifying the output file.
    parser.add_argument("-o", "--output", type=str, 
                        help="Specify the output image file.")
    # Define the argument for multiply the image by.
    parser.add_argument("-m", "--multiply", default=1.0, type=float, 
                        help="Multiple the image by.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: No input image file provided."
        sys.exit()
        
    # Check that the output parameter has been specified.
    if args.output == None:
        # Print an error message if not and exit.
        print "Error: No output image file provided."
        sys.exit()
    
    # Create input files file names associations
    infiles = applier.FilenameAssociations()
    # Set image1 to the input image specified
    infiles.image1 = args.input
    # Create output files file names associations
    outfiles = applier.FilenameAssociations()
    # Set outImage to the output image specified
    outfiles.outimage = args.output
    # Create other arguments object
    otherargs = applier.OtherInputs()
    # Define the scale arguments
    otherargs.scale = args.multiply
    # Create a controls objects
    aControls = applier.ApplierControls()
    # Set the progress object.
    aControls.progress = cuiprogress.CUIProgressBar()
    
    # Apply the multiply function.
    applier.apply(mutliplyByValue, 
                  infiles, 
                  outfiles, 
                  otherargs, 
                  controls=aControls)
    