#!/usr/bin/env python

# Import the python Argument parser
import argparse
# Import the RIOS applier interface
from rios import applier
from rios import cuiprogress
#
import numpy

# Define the applier function
def mutliplyByValue(info, inputs, outputs, otherargs):
    # Convert the input data to Float32
    # This is because the output is a float due to the 
    # divide within the NDVI calculation.
    inputs.image1 = inputs.image1.astype (numpy.float32)
    # Calculate the NDVI for the block.
    # Note. Numpy will deal with the image iterating
    #       to all the individual pixels values.
    #       within python this is very important 
    #       as python loops are slow.
    out = ((inputs.image1[otherargs.nirband]-
           inputs.image1[otherargs.redband]) 
           /
          (inputs.image1[otherargs.nirband]+
          inputs.image1[otherargs.redband]))
    # At an extra dimension to the output array.
    # The output array needs to have 3 dimensions
    # (No Bands, Y Pixels(Rows), X Pixels(Cols)
    # In this case an extra dimension representing
    # the single image band is required.
    outputs.outimage = numpy.expand_dims(out, axis=0)

# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Define the argument for specifying the output file.
    parser.add_argument("-o", "--output", type=str, 
                        help="Specify the output image file.")
    # Define the argument for specifying the red image band
    parser.add_argument("-r", "--red", type=int, 
                        help="Specifiy red band.")
    # Define the argument for specifying the NIR image band
    parser.add_argument("-n", "--nir", type=int, 
                        help="Specifiy NIR band.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: No input image file provided."
        sys.exit()
        
    # Check that the output parameter has been specified.
    if args.output == None:
        # Print an error message if not and exit.
        print "Error: No output image file provided."
        sys.exit()
    
    # Create input files file names associations
    infiles = applier.FilenameAssociations()
    # Set image1 to the input image specified
    infiles.image1 = args.input
    # Create output files file names associations
    outfiles = applier.FilenameAssociations()
    # Set outImage to the output image specified
    outfiles.outimage = args.output
    # Create other arguments object
    otherargs = applier.OtherInputs()
    # Define the red band argument
    otherargs.redband = args.red-1
    # Define the NIR band argument
    otherargs.nirband = args.nir-1
    # Create a controls objects
    aControls = applier.ApplierControls()
    # Set the progress object.
    aControls.progress = cuiprogress.CUIProgressBar()
    
    # Apply the multiply function.
    applier.apply(mutliplyByValue, 
                  infiles,
                  outfiles, 
                  otherargs, 
                  controls=aControls)
   