#!/usr/bin/env python

# Import the GDAL python library
import osgeo.gdal as gdal
# Import the python Argument parser
import argparse
# Import the System library
import sys

# A function to read a meta-data item
# from a image band
def readBandMetaData(inputFile, band, name):
    # Open the dataset in Read Only mode
    dataset = gdal.Open(inputFile, gdal.GA_ReadOnly)
    # Check that the image  has been opened.
    if not dataset is None:
        # Get the image band
        imgBand = dataset.GetRasterBand(band)
        # Check the image band was available.
        if not imgBand is None:
            # Get the meta-data value specified.
            metaData = imgBand.GetMetadataItem(name)
            # Check that it is present
            if metaData == None:
                # If not present, print error.
                print "Could not find \'", name, "\' item."
            else:
                # Else print out the metaData value.
                print name, " = \'", metaData, "\'"
        else:
            # Print out an error message.
            print "Could not open the image band: ", band
    else:
        # Print an error message if the file 
        # could not be opened.
        print "Could not open the input image file: ", inputFile
        
# A function to read a meta-data item
# from a image
def readImageMetaData(inputFile, name):
    # Open the dataset in Read Only mode 
    dataset = gdal.Open(inputFile, gdal.GA_ReadOnly)
    # Check that the image  has been opened.
    if not dataset is None:
        # Get the meta-data value specified.
        metaData = dataset.GetMetadataItem(name)
        # Check that it is present
        if metaData == None:
            # If not present, print error.
            print "Could not find \'", name, "\' item."
        else:
            # Else print out the metaData value.
            print name, " = \'", metaData, "\'"
    else:
        # Print an error message if the file 
        # could not be opened.
        print "Could not open the input image file: ", inputFile
        
# A function to read a meta-data item
# from a image band
def listBandMetaData(inputFile, band):
    # Open the dataset in Read Only mode 
    dataset = gdal.Open(inputFile, gdal.GA_ReadOnly)
    # Check that the image  has been opened.
    if not dataset is None:
        # Get the image band
        imgBand = dataset.GetRasterBand(band)
        # Check the image band was available.
        if not imgBand is None:
            # Get the meta-data dictionary
            metaData = imgBand.GetMetadata_Dict()
            # Check it has entries.
            if len(metaData) == 0:
                # If it has no entries return
                # error message.
                print "There is no image meta-data."
            else:
                # Otherwise, print out the
                # meta-data. 
                # Loop through each entry.
                for metaItem in metaData:
                    print metaItem
        else:
            # Print out an error message.
            print "Could not open the image band: ", band
    else:
        # Print an error message if the file 
        # could not be opened.
        print "Could not open the input image file: ", inputFile
        
# A function to read a meta-data item
# from a image
def listImageMetaData(inputFile):
    # Open the dataset in Read Only mode
    dataset = gdal.Open(inputFile, gdal.GA_ReadOnly)
    # Check that the image  has been opened.
    if not dataset is None:
        # Get the meta-data dictionary
        metaData = dataset.GetMetadata_Dict()
        # Check it has entries.
        if len(metaData) == 0:
            # If it has no entries return
            # error message.
            print "There is no image meta-data."
        else:
            # Otherwise, print out the
            # meta-data. 
            # Loop through each entry.
            for metaItem in metaData:
                print metaItem
    else:
        # Print an error message if the file 
        # could not be opened.
        print "Could not open the input image file: ", inputFile

# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Define the argument for specifying image band.
    parser.add_argument("-b", "--band", type=int, default=0, 
                        help="Specify image band.")
    # Define the argument for specifying meta-data name.
    parser.add_argument("-n", "--name", type=str, 
                        help="Specify the meta-data name.")
    # Define the argument for specifying whether the 
    # meta-data field should be just listed.
    parser.add_argument("-l", "--list", action="store_true", default=False, 
                        help="Specify that meta data items should be listed.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: No input image file provided."
        sys.exit()
        
    # Check that the name parameter has been specified.
    # If it has been specified then run functions to 
    # read band meta-data.
    if not args.name == None:
        # Check whether the image band has been specified.
        # the default was set at 0 (to indicate that it) 
        # hasn't been specified as GDAL band count starts
        # at 1. This also means the user cannot type in
        # a value of 0 and get an error.
        if args.band == 0:
            # Run the function to print out the image meta-data value.
            readImageMetaData(args.input, args.name)
        else:
            # Otherwise, run the function to print out the band meta-data value.
            readBandMetaData(args.input, args.band, args.name)
    elif args.list:
        if args.band == 0:
            # Run the function to list image meta-data.
            listImageMetaData(args.input)
        else:
            # Otherwise, run the function to list band meta-data.
            listBandMetaData(args.input, args.band)
    else:
        # Print an error message if not and exit.
        print "Error: the meta-data name or list option" + \
              " need to be specified was not specified."
        sys.exit()
