#!/usr/bin/env python

# Import the system library
import sys
# Import the python Argument parser
import argparse
# Import the RIOS applier interface
from rios import applier
# Import the RIOS progress feedback
from rios import cuiprogress
# Import the numpy library
import numpy
# Import the GDAL library
from osgeo import gdal

# Define the applier function
def rulebaseClassifier(info, inputs, outputs):
    # Create an output array with the same dims 
    # as a single band of the input file.
    out = numpy.zeros(inputs.image1[0].shape)
    # Use where statements to select the 
    # pixels to be classified. Give them a 
    # integer value (i.e., 1, 2, 3, 4) to 
    # specify the class.
    out[numpy.where((inputs.image1[0] > 0.4 )&(inputs.image1[0] < 0.7))] = 1
    out[numpy.where(inputs.image1[0] < 0.1 )] = 2
    out[numpy.where((inputs.image1[0] > 0.1 )&(inputs.image1[0] < 0.4))] = 3
    out[numpy.where(inputs.image1[0] > 0.7 )] = 4
    # Expand the output array to include a single
    # image band and set as the output dataset.
    outputs.outimage = numpy.expand_dims(out, axis=0)

# A function to define the image as thematic
def setThematic(imageFile):
    # Use GDAL to open the dataset
    ds = gdal.Open(imageFile, gdal.GA_Update)
    # Iterate through the image bands
    for bandnum in range(ds.RasterCount):
        # Get the image band
        band = ds.GetRasterBand(bandnum + 1)
        # Define the meta-data for the LAYER_TYPE
        band.SetMetadataItem('LAYER_TYPE', 'thematic')

# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Define the argument for specifying the output file.
    parser.add_argument("-o", "--output", type=str, 
                        help="Specify the output image file.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: No input image file provided."
        sys.exit()
        
    # Check that the output parameter has been specified.
    if args.output == None:
        # Print an error message if not and exit.
        print "Error: No output image file provided."
        sys.exit()
    
    # Create input files file names associations
    infiles = applier.FilenameAssociations()
    # Set image1 to the input image specified
    infiles.image1 = args.input
    # Create output files file names associations
    outfiles = applier.FilenameAssociations()
    # Set outImage to the output image specified
    outfiles.outimage = args.output
    # Create a controls objects
    aControls = applier.ApplierControls()
    # Specify that stats shouldn't be calc'd
    aControls.calcStats = False
    # Set the progress object.
    aControls.progress = cuiprogress.CUIProgressBar()
    
    # Apply the classifier function.
    applier.apply(rulebaseClassifier, 
                  infiles, 
                  outfiles,
                  controls=aControls)
    
    # Set the output file to be thematic
    setThematic(args.output)
