#!/usr/bin/env python

# Import the GDAL python library
import osgeo.gdal as gdal
# Import the python Argument parser
import argparse
# Import the System library
import sys

# A function to set the no data value
# for each image band.
def setNoData(inputFile, noDataVal):
    # Open the image file, in update mode
    # so that the image can be edited. 
    dataset = gdal.Open(inputFile, gdal.GA_Update)
    # Check that the image  has been opened.
    if not dataset is None:
        # Iterate throught he image bands 
        # Note. i starts at 0 while the 
        # band count in GDAL starts at 1.
        for i in range(dataset.RasterCount):
            # Print information to the user on what is 
            # being set.
            print "Setting No data (" + str(noDataVal) + ") for band " + str(i+1)
            # Get the image band
            # the i+1 is because GDAL bands
            # start with 1.
            band = dataset.GetRasterBand(i+1)
            # Set the no data value.
            band.SetNoDataValue(noDataVal)
    else:
        # Print an error message if the file 
        # could not be opened.
        print "Could not open the input image file: ", inputFile

# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Define the argument for specifying the no data value.
    parser.add_argument("-n", "--nodata", type=float, default=0, 
                        help="Specify the no data value to be set.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: not input image file provided."
        sys.exit()
    # Otherwise, run the function to set the no 
    # data value.
    setNoData(args.input, args.nodata)
