#!/usr/bin/env python

# Import the system library
import sys
# Import the RIOS rat library.
from rios import rat
# Import the python Argument parser
import argparse
# Import the numpy library
import numpy

# A function to add a colour table.
def addColourTable(imageFile):
    # Create a colour table (n,5) where
    # n is the number of classes to be 
    # coloured. The data type must be
    # of type integer.
    ct = numpy.zeros([5,5], dtype=numpy.int)
    
    # Set 0 to be Dark Mustard Yellow.
    ct[0][0] = 0   # Pixel Val
    ct[0][1] = 139 # Red
    ct[0][2] = 139 # Green
    ct[0][3] = 0   # Blue
    ct[0][4] = 255 # Opacity
    
    # Set 1 to be Dark Olive Green.
    ct[1][0] = 1   # Pixel Val
    ct[1][1] = 162 # Red
    ct[1][2] = 205 # Green
    ct[1][3] = 90  # Blue
    ct[1][4] = 255 # Opacity
    
    # Set 2 to be Royal Blue.
    ct[2][0] = 2   # Pixel Val
    ct[2][1] = 72  # Red
    ct[2][2] = 118 # Green
    ct[2][3] = 255 # Blue
    ct[2][4] = 255 # Opacity
    
    # Set 3 to be Dark Sea Green.
    ct[3][0] = 3   # Pixel Val
    ct[3][1] = 180 # Red
    ct[3][2] = 238 # Green
    ct[3][3] = 180 # Blue
    ct[3][4] = 255 # Opacity
    
    # Set 4 to be Forest Green.
    ct[4][0] = 4   # Pixel Val
    ct[4][1] = 34 # Red
    ct[4][2] = 139 # Green
    ct[4][3] = 34  # Blue
    ct[4][4] = 255 # Opacity
    
    rat.setColorTable(imageFile, ct)



# This is the first part of the script to 
# be executed.
if __name__ == '__main__':
    # Create the command line options 
    # parser.
    parser = argparse.ArgumentParser()
    # Define the argument for specifying the input file.
    parser.add_argument("-i", "--input", type=str, 
                        help="Specify the input image file.")
    # Call the parser to parse the arguments.
    args = parser.parse_args()
    
    # Check that the input parameter has been specified.
    if args.input == None:
        # Print an error message if not and exit.
        print "Error: No input image file provided."
        sys.exit()
        
    # Run the add colour table function
    addColourTable(args.input)
